/// <reference types="node" />
import { AxiosRequestConfig } from 'axios';
import { IgCookie, IChangedProfilePicture, ISearchFollow } from './types';
import { IGUserMetadata, UserGraphQL } from './types/UserMetadata';
import { IGStoriesMetadata } from './types/StoriesMetadata';
import { ReelsIds } from './types/HighlightMetadata';
import { HMedia, IHighlightsMetadata } from './types/HighlightMediaMetadata';
import { IPostModels, IRawBody } from './types/PostModels';
import { PostFeedResult } from './types/PostFeedResult';
import { PostStoryResult } from './types/PostStoryResult';
import { MediaConfigureOptions } from './types/MediaConfigureOptions';
import { UserGraphQlV2 } from './types/UserGraphQlV2';
import { IPaginatedPosts } from './types/PaginatedPosts';
export * from './utils';
export * as InstagramMetadata from './types';
export * from './helper/Session';
export declare class igApi {
    private IgCookie;
    AxiosOpts: AxiosRequestConfig;
    /**
     * Recommended to set cookie for most all IG Request
     * @param IgCookie cookie you can get it by using getSessionId function, see README.md or example file
     * @param storeCookie
     * @param AxiosOpts
     */
    constructor(IgCookie?: IgCookie, AxiosOpts?: AxiosRequestConfig);
    private accountUserId;
    private buildHeaders;
    /**
     * Make request to IG API
     * @param baseURL
     * @param url
     * @param agent
     * @param AxiosOptions
     */
    private FetchIGAPI;
    /**
     * get user id by username
     * @param {username} username
     * @returns
     */
    getIdByUsername: (username: string) => Promise<string>;
    searchFollower: (userId: string, seachTerm: string) => Promise<ISearchFollow>;
    searchFollowing: (userId: string, seachTerm: string) => Promise<ISearchFollow>;
    private _formatSidecar;
    fetchPost: (url: string) => Promise<IPostModels>;
    fetchPostByMediaId: (mediaId: string | number) => Promise<IRawBody>;
    /**
     * fetch client account profile
     */
    accountInfo: (userID?: string) => Promise<UserGraphQL>;
    /**
     * fetch profile by username. including email, phone number
     * @param {username} username
     * @returns
     */
    fetchUser: (username: string) => Promise<IGUserMetadata>;
    /**
     * hmmm..?
     * @param username
     * @returns
     */
    fetchUserV2: (username: string) => Promise<UserGraphQlV2>;
    /**
     * simple method to check is user follow me
     * @param username
     * @returns true if user is follow me
     */
    isFollowMe: (username: string) => Promise<boolean | undefined>;
    /**
     *
     * @param {StoriesGraphQL} metadata
     * @returns {ItemStories[]}
     */
    private _parseStories;
    /**
     * fetches stories metadata
     * @param {string} username username target to fetch the stories, also work with private profile if you use cookie \w your account that follows target account
     * @returns
     */
    fetchStories: (username: string) => Promise<IGStoriesMetadata>;
    /**
     * Fetch all reels/highlight id
     * @param {username} username
     * @returns
     */
    _getReelsIds: (username: string) => Promise<ReelsIds[]>;
    /**
     * get media urls from highlight id
     * @param {ids} ids of highlight
     * @returns
     */
    _getReels: (ids: string) => Promise<HMedia>;
    private formatHighlight;
    /**
     * fetches highlight metadata
     * @param {string} username username target to fetch the highlights, also work with private profile if you use cookie \w your account that follows target account
     * @returns
     */
    fetchHighlights: (username: string) => Promise<IHighlightsMetadata>;
    /**
     * fetches user posts, with pagination
     * @deprecated Does not return all information about a post, use fetchUserPostsV2()
     * @param username
     * @param end_cursor get end_cursor by fetch user posts first
     * @returns
     */
    fetchUserPosts: (username: string, end_cursor?: string) => Promise<IPaginatedPosts>;
    /**
     * fetches user posts, with pagination
     * @param username
     * @param end_cursor get end_cursor by fetchUserPostsV2 first
     * @returns
     */
    fetchUserPostsV2: (username: string, end_cursor?: string) => Promise<IPaginatedPosts>;
    private uploadPhoto;
    /**
     * Post a photo to instagram
     * @param photo file path or Buffer
     * @param type post type
     * @param options
     * @returns
     */
    addPost: (photo: string | Buffer, type: "feed" | "story" | undefined, options: MediaConfigureOptions) => Promise<PostFeedResult | PostStoryResult>;
    /**
     *
     * @param photo input must be filepath or buffer
     */
    changeProfilePicture: (photo: string | Buffer) => Promise<IChangedProfilePicture>;
}
