"use strict";
/* Muhamad Ristiyanto _ https://github.com/Gimenz
 * Created, Published at Selasa, 9 Maret 2021
 * Modified, Updated at Minggu, 20 Februari 2022
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.igApi = exports.InstagramMetadata = void 0;
const fs_1 = __importDefault(require("fs"));
const form_data_1 = __importDefault(require("form-data"));
const axios_1 = __importDefault(require("axios"));
const index_1 = require("./utils/index");
const types_1 = require("./types");
const query_1 = require("./helper/query");
const config_1 = require("./config");
const Session_1 = require("./helper/Session");
__exportStar(require("./utils"), exports);
exports.InstagramMetadata = __importStar(require("./types"));
__exportStar(require("./helper/Session"), exports);
class igApi {
    /**
     * Recommended to set cookie for most all IG Request
     * @param IgCookie cookie you can get it by using getSessionId function, see README.md or example file
     * @param storeCookie
     * @param AxiosOpts
     */
    constructor(IgCookie = '', AxiosOpts = {}) {
        var _a;
        this.IgCookie = IgCookie;
        this.AxiosOpts = AxiosOpts;
        this.accountUserId = ((_a = this.IgCookie.match(/sessionid=(.*?);/)) === null || _a === void 0 ? void 0 : _a[1].split('%')[0]) || '';
        this.buildHeaders = (agent = config_1.config.android, options) => {
            return Object.assign({ 'user-agent': agent, 'cookie': `${this.IgCookie}`, 'authority': 'www.instagram.com', 'content-type': 'application/x-www-form-urlencoded', 'origin': 'https://www.instagram.com', 'accept-language': 'id-ID,id;q=0.9,en-US;q=0.8,en;q=0.7', 'sec-fetch-site': 'same-origin', 'sec-fetch-mode': 'cors', 'sec-fetch-dest': 'empty', 'x-ig-app-id': 936619743392459, 'x-ig-www-claim': 'hmac.AR3W0DThY2Mu5Fag4sW5u3RhaR3qhFD_5wvYbOJOD9qaPjIf', 'x-instagram-ajax': 1, 'x-requested-with': 'XMLHttpRequest' }, options);
        };
        /**
         * Make request to IG API
         * @param baseURL
         * @param url
         * @param agent
         * @param AxiosOptions
         */
        this.FetchIGAPI = (baseURL, url = '', agent = config_1.config.android, AxiosOptions = {}) => {
            try {
                return (0, axios_1.default)(Object.assign(Object.assign({ baseURL,
                    url, headers: AxiosOptions.headers ? AxiosOptions.headers : this.buildHeaders(agent), method: AxiosOptions.method || 'GET' }, AxiosOptions), this.AxiosOpts));
            }
            catch (error) {
                if (axios_1.default.isAxiosError(error)) {
                    throw error.response;
                }
            }
        };
        /**
         * get user id by username
         * @param {username} username
         * @returns
         */
        this.getIdByUsername = (username) => __awaiter(this, void 0, void 0, function* () {
            const res = yield this.fetchUserV2(username);
            return res === null || res === void 0 ? void 0 : res.id;
        });
        this.searchFollower = (userId, seachTerm) => __awaiter(this, void 0, void 0, function* () {
            const res = yield this.FetchIGAPI(config_1.config.instagram_api_v1, `/friendships/${userId}/followers/?count=12&query=${seachTerm}&search_surface=follow_list_page`, config_1.config.iPhone);
            return (res === null || res === void 0 ? void 0 : res.data) || res;
        });
        this.searchFollowing = (userId, seachTerm) => __awaiter(this, void 0, void 0, function* () {
            const res = yield this.FetchIGAPI(config_1.config.instagram_api_v1, `/friendships/${userId}/following/?query=${seachTerm}`, config_1.config.iPhone);
            return (res === null || res === void 0 ? void 0 : res.data) || res;
        });
        this._formatSidecar = (data) => {
            var _a, _b, _c;
            const gql = data.items[0];
            let urls = [];
            if (gql.product_type == types_1.ProductType.CAROUSEL) {
                gql.carousel_media.forEach((v, i, a) => {
                    var _a, _b, _c;
                    urls.push({
                        id: v.id,
                        url: v.media_type == types_1.MediaType.IMAGE ? v.image_versions2.candidates[0].url : ((_a = v.video_versions) === null || _a === void 0 ? void 0 : _a[0].url) || '',
                        type: v.media_type == types_1.MediaType.IMAGE ? 'image' : 'video',
                        dimensions: {
                            height: v.media_type == types_1.MediaType.IMAGE ? v.image_versions2.candidates[0].height : ((_b = v.video_versions) === null || _b === void 0 ? void 0 : _b[0].height) || 0,
                            width: v.media_type == types_1.MediaType.IMAGE ? v.image_versions2.candidates[0].width : ((_c = v.video_versions) === null || _c === void 0 ? void 0 : _c[0].width) || 0
                        }
                    });
                });
            }
            else if (gql.product_type == types_1.ProductType.REEL) {
                urls.push({
                    id: gql.id,
                    url: gql.video_versions[0].url,
                    type: 'video',
                    dimensions: {
                        height: gql.video_versions[0].height,
                        width: gql.video_versions[0].width
                    }
                });
            }
            else if (gql.product_type == types_1.ProductType.TV) {
                urls.push({
                    id: gql.id,
                    url: gql.video_versions[0].url,
                    type: 'video',
                    dimensions: {
                        height: gql.video_versions[0].height,
                        width: gql.video_versions[0].width
                    }
                });
            }
            else if (gql.product_type == types_1.ProductType.SINGLE) {
                urls.push({
                    id: gql.id,
                    url: gql.media_type == types_1.MediaType.IMAGE ? gql.image_versions2.candidates[0].url : ((_a = gql.video_versions) === null || _a === void 0 ? void 0 : _a[0].url) || '',
                    type: gql.media_type == types_1.MediaType.IMAGE ? 'image' : 'video',
                    dimensions: {
                        height: gql.media_type == types_1.MediaType.IMAGE ? gql.image_versions2.candidates[0].height : ((_b = gql.video_versions) === null || _b === void 0 ? void 0 : _b[0].height) || 0,
                        width: gql.media_type == types_1.MediaType.IMAGE ? gql.image_versions2.candidates[0].width : ((_c = gql.video_versions) === null || _c === void 0 ? void 0 : _c[0].width) || 0
                    }
                });
            }
            return urls;
        };
        this.fetchPost = (url) => __awaiter(this, void 0, void 0, function* () {
            var _b;
            const post = (0, index_1.shortcodeFormatter)(url);
            //const req = (await IGFetchDesktop.get(`/${post.type}/${post.shortcode}/?__a=1`))
            const metadata = yield this.fetchPostByMediaId(post.media_id);
            const item = metadata.items[0];
            return {
                username: item.user.username,
                name: item.user.full_name,
                postType: (0, index_1.getPostType)(item.product_type),
                media_id: item.id,
                shortcode: item.code,
                taken_at_timestamp: item.taken_at,
                likes: item.like_count,
                caption: ((_b = item.caption) === null || _b === void 0 ? void 0 : _b.text) || null,
                media_count: item.product_type == types_1.ProductType.CAROUSEL ? item.carousel_media_count : 1,
                comment_count: item.comment_count,
                video_duration: (item === null || item === void 0 ? void 0 : item.video_duration) || null,
                music: (item === null || item === void 0 ? void 0 : item.clips_metadata) || null,
                links: this._formatSidecar(metadata),
            };
        });
        this.fetchPostByMediaId = (mediaId) => __awaiter(this, void 0, void 0, function* () {
            try {
                const res = yield this.FetchIGAPI(config_1.config.instagram_api_v1, `/media/${mediaId.toString()}/info/`);
                return res === null || res === void 0 ? void 0 : res.data;
            }
            catch (error) {
                throw error;
            }
        });
        /**
         * fetch client account profile
         */
        this.accountInfo = (userID = this.accountUserId) => __awaiter(this, void 0, void 0, function* () {
            try {
                const res = yield this.FetchIGAPI(config_1.config.instagram_api_v1, `/users/${userID}/info/`);
                const graphql = res === null || res === void 0 ? void 0 : res.data;
                return graphql;
            }
            catch (error) {
                throw error;
            }
        });
        /**
         * fetch profile by username. including email, phone number
         * @param {username} username
         * @returns
         */
        this.fetchUser = (username) => __awaiter(this, void 0, void 0, function* () {
            const userID = yield this.getIdByUsername(username);
            const res = yield this.FetchIGAPI(config_1.config.instagram_api_v1, `/users/${userID}/info/`);
            const graphql = res === null || res === void 0 ? void 0 : res.data;
            return Object.assign({ id: graphql.user.pk, username: graphql.user.username, fullname: graphql.user.full_name, followers: graphql.user.follower_count, following: graphql.user.following_count, post_count: graphql.user.media_count, is_private: graphql.user.is_private, is_verified: graphql.user.is_verified, biography: graphql.user.biography, external_url: graphql.user.external_url, total_igtv_videos: graphql.user.total_igtv_videos, has_videos: graphql.user.has_videos, hd_profile_pic_url_info: graphql.user.hd_profile_pic_url_info, has_highlight_reels: graphql.user.has_highlight_reels, has_guides: graphql.user.has_guides, is_business: graphql.user.is_business, contact_phone_number: graphql.user.contact_phone_number, public_email: graphql.user.public_email, account_type: graphql.user.account_type }, graphql);
        });
        /**
         * hmmm..?
         * @param username
         * @returns
         */
        this.fetchUserV2 = (username) => __awaiter(this, void 0, void 0, function* () {
            var _c;
            const res = yield this.FetchIGAPI(config_1.config.instagram_api_v1, `/users/web_profile_info/?username=${username}`, config_1.config.iPhone);
            const graphql = res === null || res === void 0 ? void 0 : res.data;
            return (_c = graphql.data) === null || _c === void 0 ? void 0 : _c.user;
        });
        /**
         * simple method to check is user follow me
         * @param username
         * @returns true if user is follow me
         */
        this.isFollowMe = (username) => __awaiter(this, void 0, void 0, function* () {
            const user = yield this.fetchUserV2(username);
            return user.follows_viewer;
        });
        /**
         *
         * @param {StoriesGraphQL} metadata
         * @returns {ItemStories[]}
         */
        this._parseStories = (metadata) => {
            const items = metadata.items;
            let storyList = new Array();
            for (let i = 0; i < items.length; i++) {
                if (items[i].media_type == 1) {
                    storyList.push({
                        type: 'image',
                        mimetype: 'image/jpeg',
                        url: items[i].image_versions2.candidates[0].url,
                        taken_at: items[i].taken_at,
                        expiring_at: items[i].expiring_at,
                        id: items[i].id,
                        original_width: items[i].original_width,
                        original_height: items[i].original_height,
                        has_audio: items[i].has_audio !== undefined ? items[i].has_audio : null,
                        video_duration: items[i].video_duration !== undefined
                            ? items[i].video_duration
                            : null,
                        caption: items[i].caption,
                    });
                }
                else {
                    storyList.push({
                        type: 'video',
                        mimetype: 'video/mp4',
                        url: items[i].video_versions[0].url,
                        taken_at: items[i].taken_at,
                        expiring_at: items[i].expiring_at,
                        id: items[i].id,
                        original_width: items[i].original_width,
                        original_height: items[i].original_height,
                        has_audio: items[i].has_audio !== undefined ? items[i].has_audio : false,
                        video_duration: items[i].video_duration !== undefined
                            ? items[i].video_duration
                            : null,
                        caption: items[i].caption,
                    });
                }
            }
            return storyList;
        };
        /**
         * fetches stories metadata
         * @param {string} username username target to fetch the stories, also work with private profile if you use cookie \w your account that follows target account
         * @returns
         */
        this.fetchStories = (username) => __awaiter(this, void 0, void 0, function* () {
            var _d;
            const userID = yield this.getIdByUsername(username);
            const res = yield this.FetchIGAPI(config_1.config.instagram_api_v1, `/feed/user/${userID}/reel_media/`, config_1.config.iPhone);
            const graphql = res === null || res === void 0 ? void 0 : res.data;
            const isFollowing = typeof ((_d = graphql.user) === null || _d === void 0 ? void 0 : _d.friendship_status) !== 'undefined';
            if (!isFollowing && graphql.user.is_private) {
                throw new Error('Private profile');
            }
            else {
                return {
                    username: graphql.user.username,
                    stories_count: graphql.media_count,
                    stories: graphql.items.length == 0 ? null : this._parseStories(graphql),
                    graphql,
                };
            }
        });
        /**
         * Fetch all reels/highlight id
         * @param {username} username
         * @returns
         */
        this._getReelsIds = (username) => __awaiter(this, void 0, void 0, function* () {
            const userID = yield this.getIdByUsername(username);
            const res = yield this.FetchIGAPI(config_1.config.instagram_base_url, '/graphql/query/', config_1.config.iPhone, { params: (0, query_1.highlight_ids_query)(userID) });
            const graphql = res === null || res === void 0 ? void 0 : res.data;
            let items = new Array();
            graphql.data.user.edge_highlight_reels.edges.map((edge) => {
                items.push({
                    highlight_id: edge.node.id,
                    cover: edge.node.cover_media.thumbnail_src,
                    title: edge.node.title
                });
            });
            return items;
        });
        /**
         * get media urls from highlight id
         * @param {ids} ids of highlight
         * @returns
         */
        this._getReels = (ids) => __awaiter(this, void 0, void 0, function* () {
            const res = yield this.FetchIGAPI(config_1.config.instagram_base_url, '/graphql/query/', config_1.config.iPhone, { params: (0, query_1.highlight_media_query)(ids) });
            const graphql = res === null || res === void 0 ? void 0 : res.data;
            return graphql;
        });
        this.formatHighlight = (graphql) => __awaiter(this, void 0, void 0, function* () {
            return graphql.data.reels_media[0].items.map((item) => ({
                owner: graphql.data.reels_media[0].owner,
                media_id: item.id,
                mimetype: item.is_video ? 'video/mp4' || 'video/gif' : 'image/jpeg',
                taken_at: item.taken_at_timestamp,
                type: item.is_video ? 'video' : 'image',
                url: item.is_video ? item.video_resources[0].src : item.display_url,
                dimensions: item.dimensions,
            }));
        });
        /**
         * fetches highlight metadata
         * @param {string} username username target to fetch the highlights, also work with private profile if you use cookie \w your account that follows target account
         * @returns
         */
        this.fetchHighlights = (username) => __awaiter(this, void 0, void 0, function* () {
            try {
                const ids = yield this._getReelsIds(username);
                const reels = yield Promise.all(ids.map((x) => __awaiter(this, void 0, void 0, function* () { return this.formatHighlight(yield this._getReels(x.highlight_id)); })));
                let data = [];
                for (let i = 0; i < reels.length; i++) {
                    data.push({
                        title: ids[i].title,
                        cover: ids[i].cover,
                        media_count: reels[i].length,
                        highlights_id: ids[i].highlight_id,
                        highlights: reels[i]
                    });
                }
                let json = {
                    username,
                    highlights_count: ids.length,
                    data: data
                };
                return json;
            }
            catch (error) {
                throw error;
            }
        });
        /**
         * fetches user posts, with pagination
         * @deprecated Does not return all information about a post, use fetchUserPostsV2()
         * @param username
         * @param end_cursor get end_cursor by fetch user posts first
         * @returns
         */
        this.fetchUserPosts = (username, end_cursor = '') => __awaiter(this, void 0, void 0, function* () {
            var _e;
            const userId = yield this.getIdByUsername(username);
            const params = {
                'query_id': '17880160963012870',
                'id': userId,
                'first': 12,
                'after': end_cursor
            };
            const res = yield this.FetchIGAPI(config_1.config.instagram_base_url, '/graphql/query/', config_1.config.android, { params });
            return (_e = res === null || res === void 0 ? void 0 : res.data) === null || _e === void 0 ? void 0 : _e.data.user.edge_owner_to_timeline_media;
        });
        /**
         * fetches user posts, with pagination
         * @param username
         * @param end_cursor get end_cursor by fetchUserPostsV2 first
         * @returns
         */
        this.fetchUserPostsV2 = (username, end_cursor = '') => __awaiter(this, void 0, void 0, function* () {
            var _f;
            const userId = yield this.getIdByUsername(username);
            const params = {
                'query_hash': '69cba40317214236af40e7efa697781d',
                'variables': {
                    "id": userId,
                    "first": 12,
                    "after": end_cursor
                }
            };
            const res = yield this.FetchIGAPI(config_1.config.instagram_base_url, '/graphql/query/', config_1.config.android, { params });
            return (_f = res === null || res === void 0 ? void 0 : res.data) === null || _f === void 0 ? void 0 : _f.data.user.edge_owner_to_timeline_media;
        });
        this.uploadPhoto = (photo) => __awaiter(this, void 0, void 0, function* () {
            try {
                const uploadId = Date.now();
                const file = Buffer.isBuffer(photo)
                    ? photo
                    : fs_1.default.existsSync(photo)
                        ? fs_1.default.readFileSync(photo)
                        : photo;
                const uploadParams = {
                    media_type: 1,
                    upload_id: uploadId.toString(),
                    upload_media_height: 1080,
                    upload_media_width: 1080,
                    xsharing_user_ids: JSON.stringify([]),
                    image_compression: JSON.stringify({
                        lib_name: 'moz',
                        lib_version: '3.1.m',
                        quality: '80'
                    })
                };
                const nameEntity = `${uploadId}_0_${(0, index_1.randInt)(1000000000, 9999999999)}`;
                const headers = {
                    'x-entity-type': 'image/jpeg',
                    offset: 0,
                    'x-entity-name': nameEntity,
                    'x-instagram-rupload-params': JSON.stringify(uploadParams),
                    'x-entity-length': Buffer.byteLength(file),
                    'Content-Length': Buffer.byteLength(file),
                    'Content-Type': 'application/octet-stream',
                    'x-ig-app-id': `1217981644879628`,
                    'Accept-Encoding': 'gzip',
                    'X-Pigeon-Rawclienttime': (Date.now() / 1000).toFixed(3),
                    'X-IG-Connection-Speed': `${(0, index_1.randInt)(3700, 1000)}kbps`,
                    'X-IG-Bandwidth-Speed-KBPS': '-1.000',
                    'X-IG-Bandwidth-TotalBytes-B': '0',
                    'X-IG-Bandwidth-TotalTime-MS': '0',
                };
                const headersPhoto = this.buildHeaders(config_1.config.android, headers);
                const result = yield this.FetchIGAPI(`${config_1.config.instagram_base_url}`, `/rupload_igphoto/fb_uploader_${nameEntity}`, config_1.config.android, { headers: headersPhoto, data: file, method: 'POST' });
                return result === null || result === void 0 ? void 0 : result.data;
            }
            catch (error) {
                throw error;
            }
        });
        /**
         * Post a photo to instagram
         * @param photo file path or Buffer
         * @param type post type
         * @param options
         * @returns
         */
        this.addPost = (photo, type = 'feed', options) => __awaiter(this, void 0, void 0, function* () {
            try {
                const dateObj = new Date();
                const now = dateObj
                    .toISOString()
                    .replace(/T/, ' ')
                    .replace(/\..+/, ' ');
                const offset = dateObj.getTimezoneOffset();
                const responseUpload = yield this.uploadPhoto(photo);
                const payloadForm = Object.assign({ upload_id: responseUpload.upload_id, timezone_offset: offset, date_time_original: now, date_time_digitalized: now, source_type: '4' }, options);
                let headers = {
                    'authority': 'www.instagram.com',
                    'x-ig-www-claim': 'hmac.AR2-43UfYbG2ZZLxh-BQ8N0rqGa-hESkcmxat2RqMAXejXE3',
                    'x-instagram-ajax': 'adb961e446b7-hot',
                    'content-type': 'application/x-www-form-urlencoded',
                    'accept': '*/*',
                    'user-agent': config_1.config.desktop,
                    'x-requested-with': 'XMLHttpRequest',
                    'x-csrftoken': (0, index_1.parseCookie)(this.IgCookie).csrftoken,
                    'x-ig-app-id': '1217981644879628',
                    'origin': 'https://www.instagram.com',
                    'sec-fetch-site': 'same-origin',
                    'sec-fetch-mode': 'cors',
                    'sec-fetch-dest': 'empty',
                    'referer': 'https://www.instagram.com/',
                    'accept-language': 'id-ID,id;q=0.9,en-US;q=0.8,en;q=0.7',
                    'cookie': `${this.IgCookie}`,
                };
                const result = yield this.FetchIGAPI(`${config_1.config.instagram_api_v1}`, `/media/${type === 'feed' ? 'configure/' : 'configure_to_story/'}`, config_1.config.android, { data: new URLSearchParams(Object.entries(payloadForm)).toString(), method: 'POST', headers: headers });
                return result === null || result === void 0 ? void 0 : result.data;
            }
            catch (error) {
                throw error;
            }
        });
        /**
         *
         * @param photo input must be filepath or buffer
         */
        this.changeProfilePicture = (photo) => __awaiter(this, void 0, void 0, function* () {
            const media = Buffer.isBuffer(photo) ? (0, index_1.bufferToStream)(photo) : fs_1.default.createReadStream(photo);
            const form = new form_data_1.default();
            form.append('profile_pic', media, 'profilepic.jpg');
            const headers = this.buildHeaders(config_1.config.desktop, Object.assign({ 'X-CSRFToken': yield (0, Session_1.getCsrfToken)() }, form.getHeaders()));
            const result = yield this.FetchIGAPI(config_1.config.instagram_base_url, '/accounts/web_change_profile_picture/', config_1.config.desktop, {
                method: 'post',
                data: form,
                headers
            });
            return result === null || result === void 0 ? void 0 : result.data;
        });
        this.IgCookie = IgCookie;
        this.AxiosOpts = AxiosOpts;
    }
}
exports.igApi = igApi;
